
/* Intro {{{
 * ----------------------------------------------------------------
 * DConnect Daemon
 *
 *	 #(@) Copyright (c) 2002, DConnect development team
 *	 #(@) Homepage: http://www.dc.ds.pg.gda.pl/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 * MA 02139, USA.
 *
 * ----------------------------------------------------------------
 * }}} */

/* include local header files {{{ */
#include "pch.h"
#include "main.h"
#include "conf.h"
#include "misc.h"
#include "net.h"
#include "console.h"
#include "cmd.user.h"
/* }}} */

extern userrec_t *user[MAXUSERS];	/* users table */
extern config_t conf;		/* configuration data */
extern pthread_t listen_th_cons;	/* listen thread handle for remote console */
extern pthread_mutex_t mutex;		/* provides synchronization between threads */
extern pthread_mutex_t mutex_listen_cons;
extern fd_set sockfdset;		/* descriptors */
extern int listen_cons;			/* console listening socket */
extern pthread_mutex_t mutex_inet_ntoa; // securing inet_ntoa

/* addcons() - allocate and initialize new console user {{{ */
void addcons(int sock,char *remote) {
	userrec_t *tmp;

	tmp=(userrec_t *)my_malloc(sizeof(userrec_t));
	memset(tmp,0,sizeof(userrec_t));

	user_set_state(tmp,STATE_CONNECTED);
	tmp->sock=sock;
	tmp->cons=1;
	tmp->ip=remote;

	tmp->myinfo.flag=1;

	if (setsockopts(sock,0))
		debug(DEBUG_WRNG,"setsockopts() failed.");

	/* we use select() anyway, but let's care for blocking send() */
	if (fcntl(sock,F_SETFL,O_NONBLOCK)<0) debug(DEBUG_WRNG,"fcntl() failed: %s.", strerror(errno));

	if (denied(sock))
	{
		disconnect(tmp,"You're not allowed to connect. Good bye.");
		return;
	}

	if (!validhost(tmp,conf.conf_callow,1))
	{
		disconnect(tmp,"Your host is not allowed.");
		return;
	}

	if (NUSERS>=MAXUSERS)
	{
		disconnect(tmp,"Users table is full! Increase MAXUSERS in main.h and recompile.");
		return;
	}

	tmp->nick=NULL;
	tmp->idle=time(NULL);
	my_duplicate("Console",&tmp->myinfo.ctype);

	sendf(tmp,"\r\nDConnect Daemon v%s\r\n",PACKAGE_VERSION);
	sendf(tmp,"login: ");
	user_set_state(tmp,STATE_KEY);
	pthread_mutex_lock( &mutex );

	user[NUSERS++]=tmp;
	pthread_mutex_unlock( &mutex );
} /* }}} */

/* listen_thread_cons() - listen for new connections to remote console {{{ */
void listen_thread_cons(void *args) {
	unsigned int new_sock,remote_size;
	struct sockaddr_in remote;
	char *ip;
	fd_set fdset;

	FD_ZERO(&fdset);

	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,NULL);

	remote_size=sizeof(struct sockaddr_in);
	/* accept connections */
	while(1)
	{
		remote_size=sizeof(struct sockaddr_in);
		if ((new_sock=accept(listen_cons,(struct sockaddr *)&remote,&remote_size))<0)
		{
			if (errno!=EINTR) debug(DEBUG_WRNG,"accept() failed: %s.", strerror(errno));
			continue;
		}

		pthread_mutex_lock(&mutex_listen_cons);

		debug(DEBUG_NET,"Connection to console from %s:%d.",ip=my_inet_ntoa(remote.sin_addr),ntohs(remote.sin_port));
		addcons(new_sock,ip);

		pthread_mutex_unlock(&mutex_listen_cons);
	}
	debug(DEBUG_WRNG,"listen_thread_cons(): this should be never reached!");
	pthread_cancel(listen_th_cons); /* never reached */
} /* }}} */

/* manage_cons() - proces console commands {{{ */
char manage_cons(userrec_t *usr,char *line) {
	const char *sep="\t\r\n";
	char *cmd;
	int count;
	char *__strtok_temp__;

	count=strlen(line);
	strip_telnet(usr,line,&count);
	line[count]=0;

	cmd=strtok_r(line,sep,&__strtok_temp__);

	debug(DEBUG_PROTO,"cmd '%s'",cmd);

	if (!cmd)
	{
/*		if (!user_tst_state(usr,STATE_LOGGEDIN|STATE_REGISTERED))
		{
			sendf(usr,"\377\374\001\r\n");
			user_set_state(usr,STATE_QUIT);
		}
*/
//		if (user_tst_state(usr,STATE_LOGGEDIN|STATE_REGISTERED)) sendf(usr,"> "); return 0;
		return 0;
	}

	if (!*cmd)
	{
		if (!user_tst_state(usr,STATE_LOGGEDIN|STATE_REGISTERED))
		{
			sendf(usr,"\377\374\001\r\n");

			user_set_state(usr,STATE_QUIT);
			return 1;
		}
		sendf(usr,"> "); return 0;
	}

	/* parse login */
	if (user_tst_state(usr,STATE_KEY))
	{
		//if nick is already in use
		if (nick2id(cmd)>=0) 
		{
			sendf(usr,"Your nick is lready in use\r\nPlease wait at least %ds before reconnect.\r\n",CLEANER_INTERVAL);
			user_set_state(usr,STATE_QUIT);
			debug(DEBUG_NET,"'%s' is already in use",cmd,usr->ip);
			return 1;
		}

		my_duplicate(cmd,&usr->nick);

		usr->cons=1;
		user_set_state(usr,STATE_PASSWORD);
		sendf(usr,"password:\377\373\001\r\n");

		return 0;
	}

	/* parse password */
	if (user_tst_state(usr,STATE_PASSWORD))
	{
		sendf(usr,"\377\374\001");

		if (!validnick(usr,usr->nick))
		{
			user_set_state(usr,STATE_QUIT);
			return 1;
		}

		sendf(usr,"\377\374\001\r\n");
		if (!validlogin(usr,usr->nick,cmd))
		{
			user_set_state(usr,STATE_QUIT);
			return 1;
		}

		if (usr->perm && !strchr(usr->perm,'c'))
		{
			user_set_state(usr,STATE_QUIT);
			return 1;
		}

		user_set_state(usr,STATE_LOGGEDIN);

		sendf(NULL,"$Hello %s|",usr->nick);

		dc_myinfo(NULL,usr);
		sendf(usr,"\r\nLogged in. Type `help' for a command list.\r\n\r\n> ");

		if (conf.flags & SHOW_JQ) sendf(NULL,"## %s has joined|",usr->nick);

		user_set_state(usr,STATE_REGISTERED);
		return 0;
	}

	command(usr,line);
	sendf(usr,"> ");
	return 0;
} /* }}} */

/* VIM Settings {{{
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * soft-stop-width: 4
 * c indent on
 * End:
 * vim600: sw=4 ts=4 sts=4 cindent fdm=marker
 * vim<600: sw=4 ts=4
 * }}} */

